/*
 * Copyright 2009-2010 Maarten Baert
 * maarten-baert@hotmail.com
 * http://www.maartenbaert.be/
 * 
 * This file is part of ExtremePhysics.
 * 
 * ExtremePhysics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ExtremePhysics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ExtremePhysics. If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: gm_hingejoint.cpp
 * Wrapper for ep_HingeJoint.
 */

#include "gm.h"

gmexport double ep_hingejoint_create(double world_id, double body1_id, double body2_id, double x1, double y1, double x2, double y2, double referencerotation) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_create: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_Body *body1, *body2;
	if((body1 = world->FindBody(gm_cast<unsigned long>(body1_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_create: Body %lu doesn't exist in world %lu.", gm_cast<unsigned long>(body1_id), world->GetID());
		return 0;
	}
	if((body2 = world->FindBody(gm_cast<unsigned long>(body2_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_create: Body %lu doesn't exist in world %lu.", gm_cast<unsigned long>(body2_id), world->GetID());
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->CreateHingeJoint(body1, body2, x1, y1, x2, y2, referencerotation, sizeof(gmuserdata)))==NULL) {
		return 0;
	}
	for(int i = 0; i<GM_USERVARS; ++i) {
		((gmuserdata*)(hingejoint->GetUserData()))->var[i] = 0;
	}
	return hingejoint->GetID();
}

gmexport double ep_hingejoint_destroy(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_destroy: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_destroy: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	return (world->DestroyHingeJoint(hingejoint))? 1 : 0;
}

gmexport double ep_hingejoint_exists(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_exists: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	return (world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id))!=NULL)? 1 : 0;
}

gmexport double ep_hingejoint_set_max_force(double world_id, double hingejoint_id, double maxforce) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_max_force: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_destroy: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	hingejoint->SetMaxForce(maxforce);
	return 1;
}

gmexport double ep_hingejoint_set_motor(double world_id, double hingejoint_id, double maxmotortorque, double motorvel) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_motor: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_motor: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	hingejoint->SetMotor(maxmotortorque, motorvel);
	return 1;
}

gmexport double ep_hingejoint_set_lower_limit(double world_id, double hingejoint_id, double maxlimittorque, double rotation) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_lower_limit: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_lower_limit: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	hingejoint->SetLowerLimit(maxlimittorque, rotation);
	return 1;
}

gmexport double ep_hingejoint_set_upper_limit(double world_id, double hingejoint_id, double maxlimittorque, double rotation) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_upper_limit: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_upper_limit: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	hingejoint->SetUpperLimit(maxlimittorque, rotation);
	return 1;
}

gmexport double ep_hingejoint_get_body1(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_body1: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_body1: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	return (hingejoint->GetBody1()==NULL)? 0 : hingejoint->GetBody1()->GetID();
}

gmexport double ep_hingejoint_get_body2(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_body2: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_body2: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	return (hingejoint->GetBody2()==NULL)? 0 : hingejoint->GetBody2()->GetID();
}

gmexport double ep_hingejoint_previous(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_previous: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_previous: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	return (hingejoint->GetPrevious()==NULL)? 0 : hingejoint->GetPrevious()->GetID();
}

gmexport double ep_hingejoint_next(double world_id, double hingejoint_id) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_next: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_next: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	return (hingejoint->GetNext()==NULL)? 0 : hingejoint->GetNext()->GetID();
}

gmexport double ep_hingejoint_set_uservar(double world_id, double hingejoint_id, double index, double value) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_uservar: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_set_uservar: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	int i = gm_cast<int>(index);
	if(i<0 || i>=GM_USERVARS) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "Can't set user variable of hinge joint %lu in world %lu, index is out of range.", hingejoint->GetID(), world->GetID());
		return 0;
	}
	((gmuserdata*)(hingejoint->GetUserData()))->var[i] = value;
	return 1;
}

gmexport double ep_hingejoint_get_uservar(double world_id, double hingejoint_id, double index) {
	ep_World *world;
	if((world = epmain.FindWorld(gm_cast<unsigned long>(world_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_uservar: World %lu doesn't exist.", gm_cast<unsigned long>(world_id));
		return 0;
	}
	ep_HingeJoint *hingejoint;
	if((hingejoint = world->FindHingeJoint(gm_cast<unsigned long>(hingejoint_id)))==NULL) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "ep_hingejoint_get_uservar: Hinge joint %lu doesn't exist in world %lu.", gm_cast<unsigned long>(hingejoint_id), world->GetID());
		return 0;
	}
	int i = gm_cast<int>(index);
	if(i<0 || i>=GM_USERVARS) {
		epmain.Message(EP_MESSAGELEVEL_ERROR, "Can't get user variable of hinge joint %lu in world %lu, index is out of range.", hingejoint->GetID(), world->GetID());
		return 0;
	}
	return ((gmuserdata*)(hingejoint->GetUserData()))->var[i];
}

